"""
In Django admin:
    - Use select2 (autocomplete_fields) everywhere. HowTo? in admin.py use: `import autocomplete_all as admin`
    - Add ?key=.. into Referer url (makes possible distinguish which ForeignKey calls an autocomplete) HowTo? add `autocomplete_all` into `INSTALLED_APPS`
    - Remove danger Edit/Delete buttons near to ForeignKey. HowTo? add `autocomplete_all` into `INSTALLED_APPS` before(!) `django.contrib.admin`
"""

from ._version import __version__


__version_info__ = VERSION = tuple(map(lambda x: int(x), __version__.split('.')))


# make this accessible at basic level as if it would be module (and not a package)
from django.contrib.admin import (AdminSite, AllValuesFieldListFilter, BooleanFieldListFilter, ChoicesFieldListFilter, DateFieldListFilter,
            EmptyFieldListFilter, FieldListFilter, HORIZONTAL, ListFilter, RelatedFieldListFilter, RelatedOnlyFieldListFilter, SimpleListFilter,
            VERTICAL, actions, apps, autodiscover, autodiscover_modules, checks, decorators, exceptions, filters, helpers,
            options, register, site, sites, templatetags, utils, views, widgets)
            # skipped: default_app_config, models (because with them it is impossible to add `autocomplete_all` into INSTALLED_APPS)
            # skipped ModelAdmin, StackedInline, TabularInline
from .autocomplete_all import ModelAdmin, StackedInline, TabularInline



from django.conf import settings
from django.contrib import admin


if not settings.configured:
    settings.configure()   # required for docs: make html


# this mixin was added in 0.3 because it could fix problems with missing autocomplete widget in some scenario(s)
class MediaMixin:
    # here we use files distributed inside django 2+
    class Media:
        extra = '' if settings.DEBUG else '.min'
        css = {
            "screen": (
                'admin/css/vendor/select2/select2%s.css' % extra,
                'admin/css/autocomplete.css',
            )
        }
        js = (
            'admin/js/vendor/select2/select2.full%s.js' % extra,
            'admin/js/autocomplete.js',
        )

    """
    # this is setting for dal & dal_select2

    class Media:
        css = {
            "all": (
                'vendor/select2/dist/css/select2.css',
                'autocomplete_light/select2.css',
            )
        }
        js = (
            # 'jquery/jquery-3.4.1.min.js',            # we believe jquery is loaded
            'autocomplete_light/jquery.init.js',
            'autocomplete_light/autocomplete.init.js',
            'vendor/select2/dist/js/select2.full.js',
            'autocomplete_light/select2.js',
            'autocomplete_light/forward.js',
            'autocomplete_light/jquery.post-setup.js',
        )
    """


class ModelAdmin(admin.ModelAdmin, MediaMixin):
    """
    Edit form in Admin based on this class has all related fields with autocomplete/search/select2 support.
    (except of: has defined autocomplete_fields or is marked as autocomplete_all=False)

    If used as the parent class in Admin, you will probably get few messages about missing and necessary
    Admin registrations and about missing search_fields=.. in already registered classes.
    Please make the suggested (and required) changes by hand.
    """
    autocomplete_all = True

    def __init__(self, model, *args, **kwargs):
        _modify_class(self, model)
        super().__init__(model, *args, **kwargs)


class StackedInline(admin.StackedInline, MediaMixin):
    """
    Modification of StackedInline with autocomplete for all many_to_one and many_to_many fields.
    """
    autocomplete_all = True

    def __init__(self, *args, **kwargs):
        _modify_class(self, self.model)
        super().__init__(*args, **kwargs)


class TabularInline(admin.TabularInline, MediaMixin):
    """
    Modification of TabularInline with autocomplete for all many_to_one and many_to_many fields.
    """
    autocomplete_all = True

    def __init__(self, *args, **kwargs):
        _modify_class(self, self.model)
        super().__init__(*args, **kwargs)


def _modify_class(model_admin_or_inline, model):
    """
    Fills autocomplete_fields with specified values.

    :param model_admin_or_inline: class to be modified
    :param model: model of the class
    :return: nothing (side effect - modification of class)
    """
    if model_admin_or_inline.autocomplete_all and not model_admin_or_inline.autocomplete_fields:
        autocomplete_except = getattr(model_admin_or_inline, 'autocomplete_except', [])
        acf = []
        for fld in model._meta.get_fields():
            if (fld.many_to_one or fld.many_to_many) and (fld.name not in autocomplete_except):
                acf.append(fld.name)
        if acf:
            model_admin_or_inline.autocomplete_fields = acf



#!/usr/bin/env python

"""
    edit _version.py
    python setup.py dist
    python setup.py tag
    commit/push (neccessary to update readthedocs)
    twine upload --repository-url https://test.pypi.org/legacy/ dist/*
    make sure: click a link on test.pypi and install: pip install -i https://test.pypi.org/simple/ django-admin-autocomplete-all
    not required, but if I want to have docs/_build (it is git-excluded) generated: v docs: make html
    on readthedocs.org click your name(account) and (re)build docs
    twine upload dist/*
"""

import os
import re
import sys

from setuptools import setup


exec(open("autocomplete_all/_version.py").read())


if sys.argv[-1] == 'dist':  # původně: publish, volalo setup.py včetně: upload
    try:
        import wheel
        print("Wheel version: ", wheel.__version__)
    except ImportError:
        print('Wheel library missing. Please run "pip install wheel"')
        sys.exit()
    os.system('python setup.py sdist')        # publish: + upload
    os.system('python setup.py bdist_wheel')  # publish: + upload
    sys.exit()

if sys.argv[-1] == 'tag':
    print("Tagging the version on git:")
    os.system("git tag -a %s -m 'version %s'" % (__version__, __version__))
    os.system("git push --tags")
    sys.exit()

readme = open('README.rst').read()
history = open('HISTORY.rst').read().replace('.. :changelog:', '')

setup(
    name='django-admin-autocomplete-all',
    version=__version__,
    description="""django admin: use select2 (autocomplete_fields) everywhere""",
    long_description=readme + '\n\n' + history,
    author='Mirek Zvolský',
    author_email='zvolsky@seznam.cz',
    url='https://github.com/pyutil/django-admin-autocomplete-all',
    download_url='https://github.com/pyutil/django-admin-autocomplete-all/archive/%s.tar.gz' % __version__,
    packages=[
        'autocomplete_all',
    ],
    # package_data={'templates': ['*'], 'static': ['*'],},
    include_package_data=True,
    install_requires=[
    ],
    license="MIT",
    zip_safe=False,
    keywords='django-admin-autocomplete-all',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Framework :: Django :: 2.2',
        'Framework :: Django :: 3.0',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
)
